#
# UploadReferenceWidget
# Copyright(C), 2007, Enfold Systems, Inc. - ALL RIGHTS RESERVED
#
# This software is licensed under the Terms and Conditions
# contained within the "LICENSE.txt" file that accompanied
# this software.  Any inquiries concerning the scope or
# enforceability of the license should be addressed to:
#
# Enfold Systems, Inc.
# 4617 Montrose Blvd., Suite C215
# Houston, Texas 77006 USA
# p. +1 713.942.2377 | f. +1 832.201.8856
# www.enfoldsystems.com
# info@enfoldsystems.com
#

from Testing import ZopeTestCase
from Products.CMFPlone.utils import _createObjectByType

from Products.PloneTestCase import PloneTestCase
from Products.UploadReferenceWidget.tests import URWTestCase
from Products.UploadReferenceWidget.widget import UploadReferenceWidget

from Products.UploadReferenceWidget.tests.utils import FakeRequest
from Products.UploadReferenceWidget.tests.utils import FakeFileUpload


class IntegrationTests(URWTestCase.IntegrationTestCase):

    def afterSetUp(self):
        self.doc = _createObjectByType('Document', self.portal, id='doc')
        self.image = _createObjectByType('Image', self.portal, id='image')
        self.field = self.doc.getField('relatedItems')
        self.field.isMetadata = False
        self.setRoles('Manager')
        self.request = FakeRequest()

    def uploadContentToPath(self, obj, path, destination):
        field = obj.getField('relatedItems')
        fileupload = FakeFileUpload(filename='enfold.pdf', file=self.PDF)
        field.widget = UploadReferenceWidget(startup_directory=path)
        form = {
            'relatedItems_option': 'upload',
            'relatedItems_file': [fileupload],
        }
        self.request.form.update(form)
        obj.validate(REQUEST=self.request, data=True)
        obj.processForm(REQUEST=self.request)
        expected = destination.objectValues('ATFile')
        result = obj.getRefs('relatesTo')
        self.assertEqual(expected, result)

    def testSelectExistingContentProcessWidget(self):
        self.field.widget = UploadReferenceWidget()
        form = {
            'relatedItems': [self.image.UID()],
            'relatedItems_option': 'select',
        }
        expected = [self.image.UID()], {}
        result = self.field.widget.process_form(self.doc, self.field, form,
                                                validating=False)
        self.assertEqual(expected, result)

    def testSelectMultipleExistingContentProcessWidget(self):
        video = _createObjectByType('File', self.portal, id='video')
        self.field.widget = UploadReferenceWidget()
        form = {
            'relatedItems': [self.image.UID(), video.UID()],
            'relatedItems_option': 'select',
        }
        expected = [self.image.UID(), video.UID()], {}
        result = self.field.widget.process_form(self.doc, self.field, form,
                                                validating=False)
        self.assertEqual(expected, result)

    def testSelectExistingContentCreateReference(self):
        self.field.widget = UploadReferenceWidget()
        form = {
            'relatedItems': [self.image.UID()],
            'relatedItems_option': 'select',
        }
        self.request.form.update(form)
        self.doc.validate(REQUEST=self.request, data=True)
        self.doc.processForm(REQUEST=self.request)
        expected = [self.image]
        result = self.doc.getRefs('relatesTo')
        self.assertEqual(expected, result)

    def testSelectMultipleExistingContentCreateReferences(self):
        video = _createObjectByType('File', self.portal, id='video')
        self.field.widget = UploadReferenceWidget()
        form = {
            'relatedItems': [self.image.UID(), video.UID()],
            'relatedItems_option': 'select',
        }
        self.request.form.update(form)
        self.doc.validate(REQUEST=self.request, data=True)
        self.doc.processForm(REQUEST=self.request)
        expected = [self.image, video]
        result = self.doc.getRefs('relatesTo')
        self.assertEqual(expected, result)

    def testUploadImageCreateObjectAndReference(self):
        fileupload = FakeFileUpload(filename='enfold.gif', file=self.IMG)
        self.field.widget = UploadReferenceWidget(startup_directory='test')
        self.portal.invokeFactory('Folder', id='test')
        form = {
            'relatedItems_option': 'upload',
            'relatedItems_file': [fileupload],
        }
        self.request.form.update(form)
        self.doc.validate(REQUEST=self.request, data=True)
        self.doc.processForm(REQUEST=self.request)
        expected = self.portal.test.objectValues('ATImage')
        result = self.doc.getRefs('relatesTo')
        self.assertEqual(expected, result)
        self.assertEqual(len(result), 1)
        self.assertEqual(result[0].getContentType(), 'image/gif')

    def testUploadFileCreateObjectAndReference(self):
        fileupload = FakeFileUpload(filename='enfold.pdf', file=self.PDF)
        self.field.widget = UploadReferenceWidget(startup_directory='test')
        self.portal.invokeFactory('Folder', id='test')
        form = {
            'relatedItems_option': 'upload',
            'relatedItems_file': [fileupload],
        }
        self.request.form.update(form)
        self.doc.validate(REQUEST=self.request, data=True)
        self.doc.processForm(REQUEST=self.request)
        expected = self.portal.test.objectValues('ATFile')
        result = self.doc.getRefs('relatesTo')
        self.assertEqual(expected, result)
        self.assertEqual(len(result), 1)
        # NOTE: Plone 2.5.x sets the correct content-type, application/pdf,
        # while Plone 3.0.x seems to be using application/octet-stream
        self.failUnless(result[0].getContentType().startswith('application/'))

    def testUploadKeepsCurrentReferencesForMultiValuedField(self):
        fileupload = FakeFileUpload(filename='enfold.pdf', file=self.PDF)
        self.field.widget = UploadReferenceWidget(startup_directory='test')
        self.portal.invokeFactory('Folder', id='test')
        form = {
            'relatedItems': [self.image.UID()],
            'relatedItems_option': 'upload',
            'relatedItems_file': [fileupload],
        }
        self.request.form.update(form)
        self.doc.validate(REQUEST=self.request, data=True)
        self.doc.processForm(REQUEST=self.request)
        expected = self.portal.test.objectValues() + [self.image]
        result = self.doc.getRefs('relatesTo')
        self.assertEqual(len(expected), 2)
        self.assertEqual(expected, result)

    def testUploadKeepsCurrentReferencesForMultiValuedFieldWithMultipleUploads(self):
        fileupload1 = FakeFileUpload(filename='enfold.pdf', file=self.PDF)
        fileupload2 = FakeFileUpload(filename='enfold.gif', file=self.IMG)
        self.field.widget = UploadReferenceWidget(startup_directory='test')
        self.portal.invokeFactory('Folder', id='test')
        form = {
            'relatedItems': [self.image.UID()],
            'relatedItems_option': 'upload',
            'relatedItems_file': [fileupload1, fileupload2],
        }
        self.request.form.update(form)
        self.doc.validate(REQUEST=self.request, data=True)
        self.doc.processForm(REQUEST=self.request)
        expected = self.portal.test.objectValues() + [self.image]
        result = self.doc.getRefs('relatesTo')
        self.assertEqual(len(expected), 3)
        self.assertEqual(expected, result)

    def testUploadReplaceCurrentReferencesForSingleValuedField(self):
        fileupload = FakeFileUpload(filename='enfold.pdf', file=self.PDF)
        self.field.multiValued = False
        self.field.widget = UploadReferenceWidget(startup_directory='test')
        self.portal.invokeFactory('Folder', id='test')
        form = {
            'relatedItems': self.image.UID(),
            'relatedItems_option': 'upload',
            'relatedItems_file': [fileupload],
        }
        self.request.form.update(form)
        self.doc.validate(REQUEST=self.request, data=True)
        self.doc.processForm(REQUEST=self.request)
        expected = self.portal.test.objectValues()
        result = self.doc.getRefs('relatesTo')
        self.assertEqual(expected, result)

    def testUploadCreateObjectOnParentForCurrentPathNonFolderish(self):
        self.uploadContentToPath(self.doc, '.', self.portal)

    def testUploadCreateObjectOnSelfForCurrentPathFolderish(self):
        folder = _createObjectByType('Folder', self.portal, id='folder')
        self.uploadContentToPath(folder, '.', folder)

    def testUploadCreateObjectOnParentForInvalidPathNonFolderish(self):
        self.uploadContentToPath(self.doc, '/invalid', self.portal)

    def testUploadCreateObjectOnSelfForInvalidPathFolderish(self):
        folder = _createObjectByType('Folder', self.portal, id='folder')
        self.uploadContentToPath(folder, '/invalid', folder)


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(IntegrationTests))
    suite.addTest(ZopeTestCase.FunctionalDocFileSuite(
        'widget.txt',
        optionflags=URWTestCase.OPTIONFLAGS,
        package='Products.UploadReferenceWidget.tests',
        test_class=URWTestCase.FunctionalTestCase))
    return suite
